package de.ipk.pdwExpress.curator.control.editpanel;

import java.awt.event.ActionEvent;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComboBox;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.TreeNode;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.control.viewPanel.ViewPanelControler;
import de.ipk.pdwExpress.curator.curator.AbstractCurator;
import de.ipk.pdwExpress.curator.curator.ICurator;
import de.ipk.pdwExpress.curator.util.CuratorToolKit;
import de.ipk.pdwExpress.curator.util.SwingWorker;
import de.ipk.pdwExpress.curator.view.editpanel.IEditPanelGUI;
import de.ipk.pdwExpress.curator.view.selecttree.TermTreeDialog;
import de.ipk.pdwExpress.persistence.dataAccess.ExternDbDAO;
import de.ipk.pdwExpress.persistence.dataAccess.TrmCategoryDAO;
import de.ipk.pdwExpress.persistence.dataAccess.TrmReplaceDAO;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrlType;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

/**
 * Controller class for <code>de.ipk.pdwExpress.curator.control.EditPanel</code>
 * @author Thomas Rutkowski
 *
 */
public class EditPanelControl extends ViewPanelControler{
	

	private Action addDestAction;
	private Action removeDestAction;
	private Action addSourceAction;
	private Action removeSourceAction;	
	private Action cboTrlTypeAction;
	
	public EditPanelControl(IEditPanelGUI editPanel, CuratorApp curatorApp){
		super(editPanel,curatorApp);

		getSourceTableModel().setModelEditable(true);
		getDestTableModel().setModelEditable(true);
	}


	public void init(ICurator curator){

		logger.debug("initiating EditPanelControl");
		List<TrmReplace> trmReplaces=null;
				
		if (curator != null){
			List<Term> termsToEdit = curator.getTermsToEdit();
			 trmReplaces = new Vector<TrmReplace>(termsToEdit.size());
			
			logger.debug("termsToEdit:"+termsToEdit.size());
			
			for (Term ts:termsToEdit){
				TrmReplace trl = curator.transformTerm(ts);
				
//				// associate Replacement and TrmRelace
//				trl.setReplacement(getReplacement());
//				getReplacement().getTrmReplace().add(trl);
				
				// set the EditingSession for every new Term
				for(Term td : trl.getDestinationTerms()){
					if (td.getTrmId() == null){
//						logger.debug("no id for "+td.getTrmId());
//						td.setEditingSession(getEditingSession());
						td.setTrmImportdate(new Date());
					}
				}
				trmReplaces.add(trl);
			}
		}
//		else{
//			trmReplaces=new Vector<TrmReplace>(0);
//		}
		
		
//		Set<TrmReplace> set =getReplacement().getTrmReplace();
//		List<TrmReplace> view = new Vector<TrmReplace>(set.size()); 
//		CollectionUtils.addAll(view, set.iterator());
		if (trmReplaces !=null){

			rplViewPanel.setSourceTableModel(getSourceTableModel(trmReplaces));			
		}
		
		((IEditPanelGUI) rplViewPanel).setAddSourceAction(getAddSourceAction());
		((IEditPanelGUI) rplViewPanel).setRemoveSourceAction(getRemoveSourceAction());
		((IEditPanelGUI) rplViewPanel).setRemoveDestAction(getRemoveDestAction());
		((IEditPanelGUI) rplViewPanel).setAddDestAction(getAddDestAction());
		((IEditPanelGUI) rplViewPanel).setTrlTypeSelectedAction(getCboTrlTypeAction());		
		((IEditPanelGUI) rplViewPanel).setAvailableDatabases(getAvailableDatabases());
		((IEditPanelGUI) rplViewPanel).setAvailableTrlTypes(getAvailableTrlTypes());
		
		
		rplViewPanel.setDestinationTableModel(getDestTableModel());
		rplViewPanel.addDestinationTableSelectListener(getDestinationTableSelectListener());
		
		
		TrmCategoryDAO tctDAO = new TrmCategoryDAO();
		((IEditPanelGUI) rplViewPanel).setRootCategory(tctDAO.getRootCategoryTree(CuratorApp.getConnection()));
		
		
		logger.debug("Initialisation of EditPanelControl finished. trm_replace.count:"+trmReplaces.size());
	}



	protected Vector<ExternDB> getAvailableDatabases() {
		
		ExternDbDAO edbDAO = new ExternDbDAO();
		
		List<ExternDB> listEDB = edbDAO.getAll( CuratorApp.getConnection());
		
		Vector <ExternDB> dbs = new Vector<ExternDB>();
		dbs.addAll(listEDB);
	
		return dbs;
	}
	
	protected Vector<TrlType> getAvailableTrlTypes() {
		
		TrmReplaceDAO edbDAO = new TrmReplaceDAO();
		
		List<TrlType> listEDB = edbDAO.getAllTypes( CuratorApp.getConnection());
		
		Vector <TrlType> dbs = new Vector<TrlType>();
		dbs.addAll(listEDB);
	
		return dbs;
	}
	
	
	
	protected Action getAddDestAction() {

		if (addDestAction == null){
			addDestAction= new AbstractAction ("add"){
				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					logger.debug("adding DestinationTerm to current trmReplace:"+selectTrmReplace);
					

					Term nt =new Term();
					// ! bei neuen Term keine session s
					// nt.setEditingSession(xxx);
					selectTrmReplace.getDestinationTerms().add(nt);
					
					/*
					 * fire Event to redraw table
					 * warum nicht getDestinationTableSelectListener() ?
					 * Durch Event in sourceTable wird mapping in DestinationTable neu dargestellt
					 */
					getSrcTableSelectListener().valueChanged(null);	
				}
			};
			addDestAction.setEnabled(false);
		}
		
		return addDestAction;
	}

	protected Action getRemoveDestAction() {
		if (removeDestAction == null){
			removeDestAction= new AbstractAction ("remove"){
				private static final long serialVersionUID = 1L;
				
				public void actionPerformed(ActionEvent e) {
					logger.debug("removing DestinationTerm from current sourceTerm:"+selectTrmReplace);
					
					
					int index=rplViewPanel.getSelectedDestinationIndex();
					Term termToRemove = destTableModel.getTerms().get(index);
					
										
					boolean isRemoved = selectTrmReplace.getDestinationTerms().remove(termToRemove);

					
					if (!isRemoved)
						logger.fatal("Term was not removed."+termToRemove);
					
					/*
					 * fire Event to redraw table
					 * warum nicht getDestinationTableSelectListener() ?
					 * Durch Event in sourceTable wird mapping in DestinationTable neu dargestellt
					 */
					getSrcTableSelectListener().valueChanged(null);
//					getDestinationTableSelectListener().valueChanged(null);
					
				}
			};
			removeDestAction.setEnabled(false);
		}		
		return removeDestAction;
	}
	

	protected Action getAddSourceAction() {
		if (addSourceAction ==null){
			addSourceAction= new AbstractAction("add"){

				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					logger.debug("Action: Add Source Term.");
					curatorApp.getStatusBar().setStatus("Please select a term.");
										
					
					final SwingWorker sw = new SwingWorker(){
						
						@Override
						public Object construct() {
							TrmCategoryDAO tctDao = new TrmCategoryDAO();
							TreeNode root=CuratorToolKit.createTreeNodes(tctDao.getRootCategoryTree(CuratorApp.getConnection()));
							
							TermTreeDialog dialog = new TermTreeDialog(curatorApp.getJFrame(),root);
							CuratorToolKit.centerFrame(dialog);
							dialog.setVisible(true);
							
							return dialog.getSelectedEntry();
						}

						@Override
						public void finished() {
							Term selectedTerm = (Term)get();
							
							if (selectedTerm == null){
								logger.debug("Selection canceled. term:"+selectedTerm);
								curatorApp.getStatusBar().setStatus("Selection canceled.");
							}else{
								logger.debug("Term selected. term:"+selectedTerm);
								curatorApp.getStatusBar().setStatus("Term selected."+selectedTerm);		
								
								TrmReplace trl =AbstractCurator.getEmptyTrmReplace(selectedTerm);
								
								// TODO das muesste in AbstractCurator.getEmptyTrmReplac
								// Set a default ReplaceType
								trl.setType(getAvailableTrlTypes().get(0));
								
								getSourceTableModel().addTrmReplace(trl);
							}
							
							getSourceTableModel().fireTableDataChanged();
							
						}
					};
					sw.start();					
					
					
				}
				
			};
			addSourceAction.setEnabled(true);
		}
		return addSourceAction;
	}

	protected Action getRemoveSourceAction() {
		if (removeSourceAction ==null){
			removeSourceAction= new AbstractAction("remove"){

				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					
					logger.debug("Removing source term.");
					
					int index = rplViewPanel.getSelectSourceIndex();
					if (index == -1)
						return;
					
					getSourceTableModel().remove(index);
					getDestTableModel().setTerms(new Vector<Term>(0));
					
					getViewPanel().setSelectedSourceIndex(index);
				}
			};
			removeSourceAction.setEnabled(false);
		}		
		return removeSourceAction;
	}	

//	protected Action getFinishedAction() {
//		if (finishedAction == null){
//			finishedAction= new AbstractAction ("Accept"){
//				private static final long serialVersionUID = 1L;
//				
//				public void actionPerformed(ActionEvent e) {
//					logger.debug("editing finished.");
//					
////					Replacement replacement= ReplacementFactory.getCurrentReplacement();
////					replacement.getTrmReplace().clear();
////					
////					List<TrmReplace> replaces = getSourceTableModel().getTrmReplaces();
////					for (TrmReplace replace : replaces) {
////						replacement.getTrmReplace().add(replace);
////						replace.setReplacement(replacement);
////					}
//				};
//			};
//		}		
//		return finishedAction;
//	}

//	private EditingSession getEditingSession() {
//		return null;
////		return ReplacementFactory.getCurrentEditingSession();
//		
//		
////		if (editingSession ==null){
////			editingSession=new EditingSession();
////			editingSession.setEseUsername(System.getProperty("user.name"));
////			editingSession.setEseImportdate(new Date());
////			String curatorName;
////			if (curator ==null)
////				curatorName="manually operated";
////			else
////				curatorName=curator.getClass().getName();
////			editingSession.setEseSourceDesc("Curator: "+curatorName);
//////			editingSession.setEseDesc("");
////			editingSession.setEseSuccessful('n');
////		}
////		return editingSession;
//	}
	

	
//	public void setReplacement(Replacement replacement) {
////		this.replacement=replacement;
//		
//		List<TrmReplace> replaces=null;
//		replaces = new Vector<TrmReplace>(replacement.getTrmReplace().size());
//		replaces.addAll(replacement.getTrmReplace());
//		
//		rplViewPanel.setSourceTableModel(getSourceTableModel(replaces));
//		
//	}

	
	public IEditPanelGUI getEditPanel() {
		return (IEditPanelGUI)getViewPanel();
	}


	protected ListSelectionListener getDestinationTableSelectListener() {
		if (destinationTableSelectListener == null)
			destinationTableSelectListener= new ListSelectionListener(){
				public void valueChanged(ListSelectionEvent e) {
					int row = rplViewPanel.getSelectedDestinationIndex();
					getRemoveDestAction().setEnabled(row != -1);
				}
		};
		return destinationTableSelectListener;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.curator.control.viewPanel.ViewPanelControler#getSrcTableSelectListener()
	 */
	@Override
	protected ListSelectionListener getSrcTableSelectListener() {
				
		if (srcTableSelectListener == null){
			
			final ListSelectionListener listener = super.getSrcTableSelectListener() ;
			srcTableSelectListener= new ListSelectionListener(){
					
					public void valueChanged(ListSelectionEvent e) {
						
						// Enable/Disable Actions
						getAddDestAction().setEnabled(rplViewPanel.getSelectSourceIndex() != -1);
						getRemoveSourceAction().setEnabled(rplViewPanel.getSelectSourceIndex() != -1);
	
						listener.valueChanged(e);
//						// Show Destination Terms in Table 
//						int rowIndex = rplViewPanel.getSelectSourceIndex();
//						if (rowIndex == -1)
//							return;
//						selectTrmReplace = getSourceTableModel().getTrmReplace(rowIndex);
//						logger.debug("new TrmReplace selected:"+selectTrmReplace);
//						
//						
//						// Set to List
//						Vector<Term> tl = new Vector<Term>();
//						tl.addAll(selectTrmReplace.getDestinationTerms());
//						getDestTableModel().setTerms(tl);
						
	//					getDestTableModel().setTerms(ListOrderedSet.decorate(selectTrmReplace.getDestinationTerms()).asList());
					}
			};
		}
		return srcTableSelectListener;
	}

	protected Action getCboTrlTypeAction() {
		if (cboTrlTypeAction==null){
			cboTrlTypeAction=new AbstractAction(){

				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					List<TrmReplace> trmReplaces = getSourceTableModel().getTrmReplaces();
					TrlType selected = (TrlType)((JComboBox)e.getSource()).getSelectedItem();
					for (TrmReplace replace : trmReplaces) 
						replace.setType(selected);
					getSourceTableModel().fireTableDataChanged();
				}
				
			};
		}
		return cboTrlTypeAction;
	}


	public void addTrmReplaces(List<TrmReplace> trmReplaces) {
		getSourceTableModel().clear();
		getSourceTableModel().addTrmReplaces(trmReplaces);
//		getSourceTableModel().setTrmReplaces(trmReplaces);		
	}
	
	public List<TrmReplace> getTrmReplaces() {
		return getSourceTableModel().getTrmReplaces();		
	}
	
	
}
