package de.ipk.pdwExpress.curator.control.overview;

import java.awt.event.ActionEvent;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.persistence.dataAccess.EditingSessionDAO;
import de.ipk.pdwExpress.persistence.dataAccess.HibernateUtil;
import de.ipk.pdwExpress.persistence.model.EditingSession;
import de.ipk.pdwExpress.persistence.model.Replacement;

class ExecuteRplAction extends AbstractAction implements Action {

	protected Logger logger;

	private static final long serialVersionUID = 1L;
	private final ReplacementOverviewControl controller;
	private final JComponent parentDialog ;
	
	private final static String EXEC = "{ call curator.executeReplacement(?,? )}";
	
	public ExecuteRplAction(ReplacementOverviewControl controller){
		super("Execute");
		logger=Logger.getLogger(this.getClass());
		setEnabled(false);
		this.controller=controller;
		parentDialog = (JComponent)controller.getReplaceOverviewGUI();
	}
	
	public void actionPerformed(ActionEvent e) {
		try {
			performed(e);
		} catch (Exception e1) {
			JOptionPane.showMessageDialog(parentDialog , "Error while executing replacement. See log.", "Error", JOptionPane.ERROR_MESSAGE);
			e1.printStackTrace();
		}
	}
	
	private boolean checkUniqueRpl(Replacement rpl){
		
		EditingSessionDAO eseDao = new EditingSessionDAO();
		EditingSession ese= eseDao.getByReplacementID(rpl.getRplId(), CuratorApp.getConnection());
		
		if (ese == null)
			return true;
		
		logger.debug("Replacement is not referenced by an other Editing Session.");
		
		String message="The replacement was already executed.\n Delete old editingSession and execute second time ?";
		int selectedOpt = JOptionPane.showConfirmDialog(parentDialog, message);
		if (selectedOpt ==JOptionPane.YES_OPTION){
			logger.debug("Delete old EditngSession. ese:"+ese);
			eseDao.delete(ese, CuratorApp.getConnection());
			return true;
		}
		
		
		return false;
	}
	
	private void performed(ActionEvent e) throws Exception {
		
		Replacement rpl=controller.getSelectedReplacement();
		
		
		if(!checkUniqueRpl(rpl))
			return;
		
		Session session = HibernateUtil.getSession(CuratorApp.getConnection());
		Transaction tx = session.beginTransaction();

 
		// new EditingSession (will be referenced by table OBJ_PROPERTIES_T and Replacement_T only )
		EditingSession ese = new EditingSession();
		ese.setEseUsername(System.getProperty("user.name"));
		ese.setEseSourceDesc("CuratorTool");
		ese.setEseSuccessful('n');
		ese.setEseDesc("Execution of Replacement: "+rpl.toString());
		ese.setReplacement(rpl);
		session.save(ese);
		
		Connection con = session.connection();
		CallableStatement stmt=null;
		try {
			logger.info("Executing replacement. "+EXEC);
			stmt = con.prepareCall(EXEC);
			stmt.setInt(1, rpl.getRplId());
			stmt.setInt(2,ese.getEseId());
			stmt.execute();
		} catch (SQLException e1) {
			logger.error("Error while executing statement. "+EXEC);
			e1.printStackTrace();
		}finally{
			DbUtils.closeQuietly(stmt);
		}
				
		ese.setEseSuccessful('y');
		session.saveOrUpdate(ese);
		
		tx.commit();
		session.close();
		
		JOptionPane.showMessageDialog(parentDialog, "Replacement successful executed.");
	}
}
