package de.ipk.pdwExpress.curator.control.overview;

import java.awt.event.ActionEvent;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.control.ReplacementFactory;
import de.ipk.pdwExpress.curator.control.viewPanel.ViewPanelControler;
import de.ipk.pdwExpress.curator.util.CuratorConverter;
import de.ipk.pdwExpress.curator.view.overview.IReplaceOverviewGUI;
import de.ipk.pdwExpress.persistence.dataAccess.ReplacementDAO;
import de.ipk.pdwExpress.persistence.model.Replacement;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

public class ReplacementOverviewControl extends ViewPanelControler implements ListSelectionListener{
	
	private final IReplaceOverviewGUI replaceOverviewGUI;
//	private final CuratorApp curatorApp;
	private RplOverviewTableModel tableModel;
	
	private Action executeAction;
	private Action deleteAction;
//	private ViewPanelControler viewPanelControler;
	private Action editRplAction;

	public ReplacementOverviewControl(IReplaceOverviewGUI replaceOverviewGUI, CuratorApp curatorApp){
		super( replaceOverviewGUI.getViewPanelGUI(),curatorApp);
//		logger=Logger.getLogger(this.getClass());
		this.replaceOverviewGUI=replaceOverviewGUI;
//		this.curatorApp=curatorApp;
		
		replaceOverviewGUI.setTableModel(getTableModel());
		replaceOverviewGUI.setListSelectionListener(this);
		replaceOverviewGUI.setDeleteAction(getDeleteAction());
		replaceOverviewGUI.setExecuteAction(getExecuteAction());
		replaceOverviewGUI.setLoadRplAction(getEditRplAction());
	}

	public void init(){
		loadData();
	}
	
	private void loadData() {
		logger.debug("load all Replacements.");
		ReplacementDAO dao = new ReplacementDAO();
		List<Replacement> rpl =dao.getAll( CuratorApp.getConnection());
		logger.debug("Replacements:"+rpl.size());
		getTableModel().setDataVector(rpl);		
	}

	protected RplOverviewTableModel getTableModel() {
		if (tableModel ==null){
			tableModel=new RplOverviewTableModel();
		}
		return tableModel;
	}

	protected Action getEditRplAction() {
		if (editRplAction==null){
			editRplAction= new EditRplAction(); 
		}
		return editRplAction;
	}
	
	protected Action getDeleteAction() {
		if (deleteAction==null){
			deleteAction= new RplDeleteAction(); 
		}
		return deleteAction;
	}
	
	protected Action getExecuteAction() {
		if (executeAction ==null){
			executeAction=new ExecuteRplAction(this);
		}
		return executeAction;
	}	

//	private Action getRealodAction() {
//		if (realodAction==null){
//			realodAction=new RplReloadAction();
//		}
//		return realodAction;
//	}

	public void valueChanged(ListSelectionEvent e) {
		
		int row = replaceOverviewGUI.getSelectedRow();
		
		// Enable Buttons
		getDeleteAction().setEnabled(row>-1);
		getEditRplAction().setEnabled(row>-1);
		getExecuteAction().setEnabled(row>-1);
		
		// show Mapping of current Replacement
		if (row >-1){
			Set<TrmReplace> trmReplace = getTableModel().getDataVector().get(row).getTrmReplace();
			List<TrmReplace> trmReplaceList = new Vector<TrmReplace>(trmReplace.size());
			trmReplaceList.addAll(trmReplace);
			getSourceTableModel().setTrmReplaces(trmReplaceList);
		}else{
			this.clear();
		}
	}
	
	protected Replacement getSelectedReplacement(){
		int row = replaceOverviewGUI.getSelectedRow();
		return getTableModel().getDataVector().get(row);
	}
	
	/**
	 * 
	 * @author Thomas
	 *
	 */
	private class RplDeleteAction extends AbstractAction implements Action {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public RplDeleteAction(){
			super("Delete");
			setEnabled(false);
		}
		
		public void actionPerformed(ActionEvent e) {
			
			int row = replaceOverviewGUI.getSelectedRow();
			if (row<0)
				return;
			
			//dialog
			int r =JOptionPane.showConfirmDialog(null,"Delete Replacement ?","",
					JOptionPane.YES_NO_OPTION);
			
			if ( r==JOptionPane.NO_OPTION)
				return;
			
			Replacement rpl = getTableModel().getDataVector().get(row);
				
			logger.info("Deleteting Replacement. rpl_id:"+rpl.getRplId());
			
			ReplacementDAO dao = new ReplacementDAO();
			dao.delete(rpl, CuratorApp.getConnection());
			loadData();

		}

	}//class RplDeleteAction
	
	
	
//	/**
//	 * 
//	 * @author Thomas
//	 *
//	 */
//	private class RplReloadAction extends AbstractAction implements Action {
//		
//		public RplReloadAction(){
//			super("Reload");
//		}
//
//		public void actionPerformed(ActionEvent e) {
//			logger.debug("Reloading Replacements");
//			reloadData();
//		}
//
//	}//RplReloadAction	
	
	class EditRplAction extends AbstractAction {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public EditRplAction(){
			super("Edit Replacement");
			setEnabled(false);
		}
		
		public void actionPerformed(ActionEvent e) {
//			ReplacementFactory.setCurrentReplacement(getViewPanelControler().getReplacement());
//			Set<TrmReplace> trmReplaces = getViewPanelControler().getReplacement().getTrmReplace();
//			
			// Clear the edit panel
			curatorApp.getCuratorActions().getEditControl().clear();
			
			// get The selected Replacement
			Replacement rpl=getSelectedReplacement();
						
//			int row = replaceOverviewGUI.getSelectedRow();
//			Set<TrmReplace> trmReplace = getTableModel().getDataVector().get(row).getTrmReplace();
			
			curatorApp.getCuratorActions().getEditControl().addTrmReplaces(CuratorConverter.toList(rpl.getTrmReplace()));
			
			ReplacementFactory.setCurrentReplacement(rpl);

			// move to editpanel
			curatorApp.getCuratorActions().getEmptyCuratorAction().actionPerformed(null);
		}
	}



	protected IReplaceOverviewGUI getReplaceOverviewGUI() {
		return replaceOverviewGUI;
	}	

//	private ViewPanelControler getViewPanelControler() {
//		if (viewPanelControler == null){
//			viewPanelControler =  new ViewPanelControler(replaceOverviewGUI.getViewPanelGUI(),curatorApp);
////			editPanelControl.init();
//		}
//		return viewPanelControler;
//	}



}
