package de.ipk.pdwExpress.curator.control.viewPanel;

import java.util.List;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmCategory;

public class TermTableModel extends AbstractTableModel {
	
	protected Logger logger;

	/**
	 * 
	 */
	private static final long serialVersionUID = -8924224983999039021L;

	private boolean modelEditable=false;
	
	private List<Term> terms= new Vector <Term>(0);

	public TermTableModel(){
		logger = Logger.getLogger(this.getClass());
	}
	
	/**
	 * 
	 * @param editable
	 */
	public TermTableModel(boolean editable){
		this();
		this.modelEditable=editable;
	}
	
	

	public int getColumnCount() {
		return 6;
	}

	public int getRowCount() {
		return terms.size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex){
		case 0:
			return terms.get(rowIndex).getTrmId();//"TRM_ID";
		case 1 :
			return terms.get(rowIndex).getTrmCategory() ;//"CATEGORY";
		case 2:
			return terms.get(rowIndex).getTrmValue() ;//"VALUE";
		case 3:
			return terms.get(rowIndex).getExterndb();
		case 4:
			return terms.get(rowIndex).getTrmDesc() ;//"VALUE_DESC";
		case 5:
			return terms.get(rowIndex).getTrmEdbacc() ;//"Accession";
		default: return "error";
		
		}
	}


	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		
//		if (columnIndex==0)
//			return true;
		
		if (terms.get(rowIndex).getTrmId() != null)
			return false;
		
		return modelEditable;
	}


	@Override
	public String getColumnName(int column) {
		
		switch (column){
		case 0:
			return "TRM_ID";
		case 1 :
			return "CATEGORY";
		case 2:
			return "VALUE";
		case 3:
			return "DATABASE";
		case 4:
			return "VALUE_DESC";
		case 5:
			return "ACCESSION";
		default: return "error";
		
		}
		
	}
	
	

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
					
		logger.debug("setValueAt() rowIndex:"+rowIndex+" columnIndex:"+columnIndex+" aValue:"+aValue);
		Term t =terms.get(rowIndex);
		
		switch (columnIndex){
		case 0 :
			if (aValue instanceof Term){
				
				terms.set(rowIndex, (Term)aValue);
				
//				Term newTerm = (Term)aValue;
//				t.setTrmId(newTerm.getTrmId()); 
//				setValueAt(newTerm.getTrmCategory(),rowIndex,1);
//				setValueAt(newTerm.getTrmValue(),rowIndex,2);
				setValueAt( ((Term)aValue).getExterndb(),rowIndex,3);
//				setValueAt(newTerm.getTrmDesc(),rowIndex,4);
//				setValueAt(newTerm.getTrmEdbacc(),rowIndex,5);
				
			}else{
				Class cl = aValue==null ? null :aValue.getClass();
				logger.fatal("Unsupported Type. type:"+cl+" aValue:"+aValue);
			}
			break;
		case 1 :
			t.setTrmCategory((TrmCategory) aValue);// "CATEGORY";
			break;
		case 2:
			t.setTrmValue((String)aValue);// "VALUE";
			break;
		case 3:
			if (aValue == null)
				t.setExterndb(null);
			else if (aValue instanceof ExternDB){
				t.setExterndb((ExternDB) aValue);
				if (t.getExterndb().getEdbName().equalsIgnoreCase("unreferenced"))
					t.setTrmEdbacc("unreferenced");
			}else 
				logger.fatal("Illegal Type (ExternDB expected)."+aValue);
			break;
		case 4:
			t.setTrmDesc((String)aValue);// "VALUE_DESC";
			break;
		case 5:
			t.setTrmEdbacc((String)aValue);
			break;
		default: logger.warn("setValueAt ignores Value. value:"+aValue+ " rowIndex:"+rowIndex+ " columnIndex:"+columnIndex);
		
		}		
		
		fireTableDataChanged();
//		super.setValueAt(aValue, rowIndex, columnIndex);
	}


	@Override
	public Class<?> getColumnClass(int columnIndex) {

		switch (columnIndex) {
		case 0: 
			return Integer.class;
		case 1:
			return TrmCategory.class;
		case 3:
			return ExternDB.class;
		default:
			return super.getColumnClass(columnIndex);
		}
	}
	
	public List<Term> getTerms() {
		return terms;
	}


	public boolean isModelEditable() {
		return modelEditable;
	}

	public void addTerm(Term term) {
		logger.debug("addTerm:"+term);
		terms.add(term);
		fireTableRowsInserted(terms.size(), terms.size());		
	}

	public void setTerms(List<Term> terms) {
		this.terms = terms;
		fireTableDataChanged();
	}
	
	public Term getTerm(int i){
		return terms.get(i);
	}
	
	public Term removeTerm(int i){
		Term t =terms.remove(i);
		fireTableDataChanged();
		return t;
	}
	
	public boolean removeTerm(Term term){
		boolean b = terms.remove(term);
		fireTableDataChanged();
		return b;
	}

	public void setModelEditable(boolean modelEditable) {
		this.modelEditable = modelEditable;
	}

	public void clear() {
		terms.clear();		
	}	
	
	

}
