package de.ipk.pdwExpress.curator.control.viewPanel;

import java.util.List;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.apache.commons.collections.list.SetUniqueList;
import org.apache.log4j.Logger;

import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrlType;
import de.ipk.pdwExpress.persistence.model.TrmCategory;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

public class TrmReplaceTableModel extends AbstractTableModel {
 
	private static final long serialVersionUID = 1L;
	protected Logger logger;
	private List<TrmReplace> trmReplacesIntern =new Vector<TrmReplace>(0);
	private SetUniqueList trmReplaces;
	
	private boolean modelEditable = true;
	
	public TrmReplaceTableModel(){
		logger=Logger.getLogger(this.getClass());
		
		trmReplaces=SetUniqueList.decorate(trmReplacesIntern);
	}	
	
	
	public int getColumnCount() {
		return 7;
	}

	public int getRowCount() {
		return trmReplaces.size();
	}
	
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {

		if (columnIndex == 0) {

			if (aValue == null) {
				((TrmReplace) trmReplaces.get(rowIndex)).setType(null);
				
			} else if (aValue instanceof TrlType) {
				TrlType type = (TrlType) aValue;
				((TrmReplace) trmReplaces.get(rowIndex)).setType(type);
				
			} else {
				logger.fatal("Unexpected Type. type:" + aValue.getClass()+ " aValue:" + aValue);
			}
		}
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		Term term = ((TrmReplace) trmReplaces.get(rowIndex)).getSourceTerm();
		
		switch (columnIndex){
		case 0:
			return ((TrmReplace) trmReplaces.get(rowIndex)).getType();
		case 1:
			return term.getTrmId();//"TRM_ID";
		case 2 :
			return term.getTrmCategory() ;//"CATEGORY";
		case 3:
			return term.getTrmValue() ;//"VALUE";
		case 4:
			return term.getExterndb();
		case 5:
			return term.getTrmDesc() ;//"VALUE_DESC";
		case 6:
			return term.getTrmEdbacc() ;//"Accession";
		default: return "error";
		
		}
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return modelEditable && columnIndex ==0;
	}


	@Override
	public String getColumnName(int column) {
		
		switch (column){
		case 0:
			return "REPLACE TYPE";
		case 1:
			return "TRM_ID";
		case 2 :
			return "CATEGORY";
		case 3:
			return "VALUE";
		case 4:
			return "DATABASE";
		case 5:
			return "VALUE_DESC";
		case 6:
			return "ACCESSION";
		default: return "error";
		
		}	
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex) {

		switch (columnIndex) {
		case 0:
			return TrlType.class;
		case 1: 
			return Integer.class;
		case 2:
			return TrmCategory.class;
		case 3:
			return String.class;			
		case 4:
			return ExternDB.class;
		default:
			return super.getColumnClass(columnIndex);
		}
	}

	public List<TrmReplace> getTrmReplaces() {
		return trmReplaces;
	}

	public void setTrmReplaces(List<TrmReplace> name) {
		
		this.trmReplaces.clear();
		this.trmReplaces.addAll(name);
		fireTableDataChanged();
	}
	
	public void addTrmReplaces(List<TrmReplace> trmReplaces) {
		this.trmReplaces.addAll(trmReplaces);
		fireTableDataChanged();
	}		
	
	public TrmReplace getTrmReplace(int i){
		return (TrmReplace) trmReplaces.get(i);		
	}

	public void addTrmReplace(TrmReplace trl) {
		trmReplaces.add(trl);
		fireTableDataChanged();
	}

	public TrmReplace remove(int index) {
		TrmReplace t =(TrmReplace) trmReplaces.remove(index);
		fireTableDataChanged();
		return t; 		
	}

	public boolean isModelEditable() {
		return modelEditable;
	}

	public void setModelEditable(boolean modelEditable) {
		this.modelEditable = modelEditable;
	}



	public void clear() {
		trmReplaces.clear();		
	}
	

}
