package de.ipk.pdwExpress.curator.control.viewPanel;

import java.util.List;
import java.util.Vector;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.view.viewPanel.IViewPanelGUI;
import de.ipk.pdwExpress.persistence.model.Replacement;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

/**
 * Controller class for <code>de.ipk.pdwExpress.curator.control.ViewPanel</code>
 * 
 * @author Thomas Rutkowski
 *
 */
public class ViewPanelControler {

	protected Logger logger;

	protected final IViewPanelGUI rplViewPanel;
	protected final CuratorApp curatorApp;
	
//	private Replacement replacement;
	
	protected TermTableModel destTableModel;
	protected TrmReplaceTableModel srcTableModel;
	protected ListSelectionListener destinationTableSelectListener;
	protected ListSelectionListener srcTableSelectListener;

	protected TrmReplace selectTrmReplace;	

	
	public ViewPanelControler(IViewPanelGUI viewPanel, CuratorApp curatorApp){
		this.logger = Logger.getLogger(this.getClass());
		this.rplViewPanel=viewPanel;
		this.curatorApp=curatorApp;
		
		rplViewPanel.addSourceTableSelectListener(getSrcTableSelectListener());
		rplViewPanel.setSourceTableModel(getSourceTableModel());
		rplViewPanel.setDestinationTableModel(getDestTableModel());
	}	
	




	protected ListSelectionListener getSrcTableSelectListener() {
		
		if (srcTableSelectListener == null)
			srcTableSelectListener= new ListSelectionListener(){
				
				public void valueChanged(ListSelectionEvent e) {
					
					// Show Destination Terms in Table 
					int rowIndex = rplViewPanel.getSelectSourceIndex();
					if (rowIndex == -1)
						return;
					selectTrmReplace = getSourceTableModel().getTrmReplace(rowIndex);
					logger.debug("new TrmReplace selected:"+selectTrmReplace);
					
//					// Set to List
//					Vector<Term> tl = new Vector<Term>();
//					tl.addAll(selectTrmReplace.getDestinationTerms());
//					getDestTableModel().setTerms(tl);
					
					getDestTableModel().setTerms(selectTrmReplace.getDestinationTerms());
				}
		};
		return srcTableSelectListener;
	}

	protected TrmReplaceTableModel getSourceTableModel(List<TrmReplace> replaces) {
		if (srcTableModel ==null)
			srcTableModel= new TrmReplaceTableModel();	
		
		srcTableModel.addTrmReplaces(replaces);
		
		return srcTableModel;
	}
	
	protected TrmReplaceTableModel getSourceTableModel() {
		if (srcTableModel ==null){
			srcTableModel= new TrmReplaceTableModel();
			srcTableModel.setModelEditable(false);
		}
		
		return srcTableModel;
	}

//	
//	/**
//	 * Not Editable Model
//	 * @return
//	 */
//	protected TermTableModel getDestTableModel() {
//		return getDestTableModel(true);
//	}
	
	/**
	 * 
	 * @param editable
	 * @return
	 */
	protected TermTableModel getDestTableModel() {
		if (destTableModel ==null){
			destTableModel= new TermTableModel();
			destTableModel.setModelEditable(false);
		}
				
		return destTableModel;
	}		

	void setReplacement(Replacement replacement) {
//		this.replacement=replacement;
				
		List<TrmReplace> replaces=null;
		replaces = new Vector<TrmReplace>(replacement.getTrmReplace().size());
		replaces.addAll(replacement.getTrmReplace());
		
		rplViewPanel.setSourceTableModel(getSourceTableModel(replaces));

	}
	

	public void clear() {
		getSourceTableModel().clear();
		getSourceTableModel().fireTableDataChanged();
		getDestTableModel().clear();
		getDestTableModel().fireTableDataChanged();		
	}
	
		
//
//	public static Replacement createNewReplacement() {
//
//		Replacement rpl;
//		rpl=new Replacement();
//		rpl.setRplCreationdate(new Date());
//		rpl.setRplAuthor(System.getProperty("user.name"));
//		
//		return rpl;
//	}

	protected IViewPanelGUI getViewPanel() {
		return rplViewPanel;
	}


//	Replacement getReplacement() {
//		return replacement;
//	}

}
