package de.ipk.pdwExpress.curator.curator;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.persistence.dataAccess.ExternDbDAO;
import de.ipk.pdwExpress.persistence.dataAccess.TermDAO;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

public class AccessionCurator extends AbstractCuratorUsingOls {

	
	private Map <String, ExternDB> edbMap = new HashMap <String, ExternDB>();
	
	
	public AccessionCurator(){
	}
	
	@Override
	public List<Term> getTermsToEdit() {
		
		List<Term> allTerms;
		TermDAO tDAO = new TermDAO();

		/*
		 * max 3 letters following by
		 * colon following by 
		 * mx 7 digit
		 */
		allTerms =tDAO.getBySQLWhere("regexp_like(trm_value,'[^[:alpha:]][[:alpha:]]{3}:[[:digit:]]{7}[^[:digit:]]','i') and trm_edb_id = 0",CuratorApp.getConnection());

		return allTerms;
	}

	@Override
	public TrmReplace transformTerm(Term termSource) {
		TrmReplace tr = new TrmReplace();
		tr.setSourceTerm(termSource);
		
		Pattern p = Pattern.compile("(.*)[^A-Z]\\(?([A-Z]{2,4}:\\d{5,10})\\)?(.*)");
		Matcher m = p.matcher(termSource.getTrmValue().trim());
		
		if (m.matches()){
			Term t=null;
			
			String trmEdbacc=m.group(2);
			t = getTermByAcc(trmEdbacc);
			
			if (t==null){
//				String trmValue=m.group(1)+m.group(3);
//				ExternDB externdb=getExterDB(mapDBAccession(trmEdbacc));
//				
//				t= new Term();
//				t.setTrmEdbacc(trmEdbacc);
//				t.setTrmValue(trmValue);
//				t.setExterndb(externdb);
//				t.setTrmCategory(termSource.getTrmCategory());
			}
			
			if(t!=null){
				
				t.setTrmCategory(termSource.getTrmCategory());
				
				List<Term> destinationTerms = new Vector<Term>();
				destinationTerms.add(t);
				tr.setDestinationTerms(destinationTerms );
			}
			
		}else{
			logger.warn("Term does not match pattern. Term:"+termSource);
		}
		
		return tr;
	}
	
	
	private Term getTermByAcc(String trmEdbacc) {
		try {
			
			Term t= getOlsDaoImpl().getTermByAccession(trmEdbacc);
			
			if (t==null){
				logger.warn("Accession not Found. acc:"+trmEdbacc);
				return null;
			}
			
			String shortName=trmEdbacc.substring(0,trmEdbacc.indexOf(':'));
			ExternDbDAO edbDAO = new ExternDbDAO();
			ExternDB edb = edbDAO.getByShortName(shortName,CuratorApp.getConnection());
			t.setExterndb(edb);
			
			return t;
			
		} catch (SQLException e) {
			logger.error(e);
			e.printStackTrace();
		}
		return null;
	}

	public String getName() {
		return "Accession Curator";
	}

	public String getShortDescription() {
		return "Searches for term where content seems to contain a Accession.";
	}



	
	
//	/**
//	 * 
//	 * @param shortName Name of the DB vom EDB_NAME
//	 * @return
//	 */
//	private ExternDB getExterDB(String accession){
//		
//		String shortName=null;
//			
//		List<ExternDB> edbs;
//		ExternDbDAO edbDAO = new ExternDbDAO();
//		edbs = edbDAO.getByShortName(shortName);
//		if (!edbs.isEmpty())
//			return edbs.get(0);	
//		else
//			return null;
//	}	
//	
//	/**
//	 * 
//	 * @param accession Accession of DB e.g. GO:123,PO:4564
//	 * @return DatabaseName e.g. 'Plant Ontology'
//	 */
//	private String mapDBAccession(String accession){
//		String edbName=null;
//		
//		if (accession.startsWith("GRO")){
//			edbName="GRAMENE";
//		}else if ( accession.startsWith("PO")){
//			edbName="Plant Ontology";
//		}
//		
//		return edbName;
//		
//	}

}
