package de.ipk.pdwExpress.curator.curator;

import java.util.List;

import javax.swing.JPanel;

import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

/**
 * Provides the interface to write your own curator.
 * 
 * Register your Curator within the 'curator.properties' file.
 * 
 * @author Thomas Rutkowski
 *
 */
public interface ICurator {
	
	/**
	 * executed after the DB-Connection was created.
	 *
	 */
	public abstract void initiate();
	
	/**
	 * executed when the application is closed.
	 *
	 */
	public abstract void close();
	
	
	/**
	 * 
	 * @return All terms that should be edited. Never NULL.
	 */
	public abstract List<Term> getTermsToEdit();

	/**
	 * 
	 * @param termSource Term which should be replaced
	 * @return Never NULL. If no transformation was possible a TrmReplace with empty Set of targets.
	 */
	public abstract TrmReplace transformTerm(Term termSource);
	
	
	
	/**
	 * 
	 * @return A description of what the curator does. (Shown in the header)
	 */
	public abstract String getShortDescription();
	
	/**
	 * 
	 * @return The Name of the curator shown in the applaction.
	 */
	public abstract String getName();
			
	
	/**
	 * 
	 * @return 
	 */
	public abstract String getSuccessfulLoadedMessage();
	
	/**
	 * 
	 * @return A Panel where the user may edit settings of the curator. <br>
	 * If NULL, no config Panel will be available
	 */
	public abstract JPanel getConfigPanel();
}
