package de.ipk.pdwExpress.curator.tasks;

import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.List;
import java.util.Vector;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.control.editpanel.EditPanelControl;
import de.ipk.pdwExpress.curator.curator.AbstractCurator;
import de.ipk.pdwExpress.curator.curator.EmptyCurator;
import de.ipk.pdwExpress.curator.view.editpanel.EditPanelGUIImpl;
import de.ipk.pdwExpress.curator.view.editpanel.IEditPanelGUI;

public class CuratorActions {

	protected Logger logger;
	protected final CuratorApp curatorApp;
	
//	private Action ageCuratorAction;
//	private Action accessionCuratorAction;
	private DefaultCuratorAction emptyCuratorAction;
//	private Action organismPartCuratorAction;
//	private Action unrefDuplicateCurator;
	private final EditPanelControl editControl;
	private final IEditPanelGUI editPanelGUI = new EditPanelGUIImpl();

	private final List<DefaultCuratorAction> curators = new Vector<DefaultCuratorAction>();
	
	public CuratorActions(CuratorApp curatorApp){
		logger=Logger.getLogger(this.getClass());
		
		this.curatorApp=curatorApp;
		editControl = new EditPanelControl(editPanelGUI, curatorApp);
		
		try {
			PropertiesConfiguration config = new PropertiesConfiguration();
			
			InputStream in = getClass().getResourceAsStream("/curator.properties");
			if (in == null){
				logger.fatal("curator.properties not found.");
			}
			config.load(in , null);
						
			String[] classes = config.getStringArray("curator.class");
			
			initCuratorActions(classes);

		} catch (ConfigurationException e) {
			logger.fatal("Error while reading properties file.");
			e.printStackTrace();
		}		
		
		
	}
	
	/**
	 * Initiates the given CuratorClasses <br>
	 * Wrappes Curator in <code> DefaultCuratorAction </code> and adds Action to intern list <code> curators </code>
	 * @param classes
	 */
	private final void initCuratorActions(String[] classes )  {

		curators.add(getEmptyCuratorAction());	

		for (String clazz: classes) {
			String fullName = clazz;
			
			try {
								
				Class currentClass = Class.forName(fullName);
				if (currentClass ==null){
					logger.fatal("Did not find CuratorClass. class:'"+fullName+"'");
					continue;
				}
										
				Constructor con= currentClass.getConstructor();
				
				AbstractCurator curator=(AbstractCurator) con.newInstance();
				curators.add(new DefaultCuratorAction(curatorApp,editControl, curator));
				
			} catch (Exception e) {
				logger.fatal("Can't initiate CuratorClass. class:"+fullName);
				e.printStackTrace();
			} 
			
			logger.info("Curator initiated: "+fullName);
			
		}
	}
	
	public List<DefaultCuratorAction> getCurators() {
		return curators;
	}
	
	public DefaultCuratorAction getEmptyCuratorAction() {
		if (emptyCuratorAction == null)
			emptyCuratorAction=new DefaultCuratorAction(curatorApp, editControl,new EmptyCurator());
		return emptyCuratorAction;
	}
	
	
	
	
//
//	public Action getOrganismPartCuratorAction() {
//		if (organismPartCuratorAction ==null){
//			organismPartCuratorAction=new DefaultCuratorAction(curatorApp,editControl, new OrganismPartCurator());
//				
//			};
//		return organismPartCuratorAction;
//	}
//

//
//
//	public Action getAccessionCuratorAction() {
//		if (accessionCuratorAction == null){
//			accessionCuratorAction = new DefaultCuratorAction(curatorApp, editControl,new AccessionCurator());
//		}
//		
//		return accessionCuratorAction;
//	}
//	
//	public Action getAgeCuratorAction() {
//		if (ageCuratorAction == null){
//			ageCuratorAction = new DefaultCuratorAction(curatorApp, editControl,new AgeCurator());
//		}		
//		return ageCuratorAction;
//	}
//
	public EditPanelControl getEditControl() {
		return editControl;
	}
//
//	public Action getUnrefDuplicateCurator() {
//		if (unrefDuplicateCurator ==null){
//			unrefDuplicateCurator =new DefaultCuratorAction(curatorApp, editControl,new UnrefDuplicateCurator());
//		}
//		return unrefDuplicateCurator;
//	}
//


	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
//	/**
//	 * Switch to AgeCurator
//	 * @author Thomas
//	 *
//	 */
//	private class AgeCuratorAction extends AbstractAction {
//		private final ICurator cr = new AgeCurator();
//		
//		public AgeCuratorAction(){
////			super("Age Curator");
////			putValue(Action.SHORT_DESCRIPTION, "Searches for term where category is 'Age'.");
//			
//			putValue(Action.SHORT_DESCRIPTION,cr.getShortDescription());
//			putValue(Action.NAME,cr.getName() );
//		}
//		
//		public void actionPerformed(ActionEvent e) {
//
//			final IEditPanelGUI editPanelView = new EditPanelGUIImpl();
//			curatorApp.getStatusBar().setProgressIndeterminate(true);
//			curatorApp.setMainPanel((JPanel) editPanelView,this);
//			
//			final SwingWorker sw = new SwingWorker(){
//
//				@Override
//				public Object construct() {
//					
////					ICurator cr = new AgeCurator();
//					EditPanelControl ec= new EditPanelControl(editPanelView,cr,curatorApp);
//					ec.init();	
//					
//					return null;
//				}
//
//				@Override
//				public void finished() {
//					curatorApp.getStatusBar().setProgressIndeterminate(false);
//					curatorApp.getStatusBar().setStatus("Terms with 'Age' terms loaded.");					
//				}
//			};
//			sw.start();
//		}
//
//	}
//	
//
//	
//	/**
//	 * Switch to AccessionCurator
//	 * @author Thomas
//	 *
//	 */
//	private class AccessionCuratorAction extends AbstractAction {
//		
//		private final ICurator cr = new AccessionCurator();
//		public AccessionCuratorAction(){
////			super("Accession Curator");
////			putValue(Action.SHORT_DESCRIPTION,
////							"Searches for term where content seems to contain a Accession.");
//			
//			putValue(Action.SHORT_DESCRIPTION,cr.getShortDescription());
//			putValue(Action.NAME,cr.getName() );
//		}
//		public void actionPerformed(ActionEvent e) {
//			
//			final IEditPanelGUI editPanelView = new EditPanelGUIImpl();
//			curatorApp.getStatusBar().setProgressIndeterminate(true);
//			curatorApp.setMainPanel((JPanel) editPanelView,this);
//			
//			final SwingWorker sw = new SwingWorker(){
//				@Override
//				public Object construct() {
//					
//					EditPanelControl ec= new EditPanelControl(editPanelView,cr,curatorApp);
//					ec.init();
//					
//					return null;
//				}
//
//				@Override
//				public void finished() {
//					curatorApp.getStatusBar().setProgressIndeterminate(false);
//					curatorApp.getStatusBar().setStatus("Terms with database accessions loaded.");
//				}
//			};
//			sw.start();
//		}
//	}
//	
//	
//	
//	private class EmptyCuratorAction extends AbstractAction {
//		private final ICurator cr = new EmptyCurator();
//		public EmptyCuratorAction(){
////			super("Manual Curator");
////			putValue(Action.SHORT_DESCRIPTION,
////							"For manual editing. Does not return a edit list and proposes no Replacement.");
//			putValue(Action.SHORT_DESCRIPTION,cr.getShortDescription());
//			putValue(Action.NAME,cr.getName() );
//		}
//
//		public void actionPerformed(ActionEvent e) {
//
//			final IEditPanelGUI editPanelView = new EditPanelGUIImpl();
//
//			curatorApp.getStatusBar().setProgressIndeterminate(true);
//			curatorApp.setMainPanel((JPanel) editPanelView, this);
//			
//			final SwingWorker sw = new SwingWorker(){
//				@Override
//				public Object construct() {
////					ICurator cr = new EmptyCurator();
//					EditPanelControl ec= new EditPanelControl(editPanelView,cr,curatorApp);
//					ec.init();
//					
//					return null;
//				}
//
//				@Override
//				public void finished() {
//					curatorApp.getStatusBar().setProgressIndeterminate(false);
//					curatorApp.getStatusBar().setStatus("Empty curator initiated.");
//				}
//			};
//			sw.start();
//
//		}
//
//	}//class
//	
//
//	public class OrganismPartCuratorAction extends AbstractAction {
//
//		private ICurator cr = new OrganismPartCurator();;
//		public OrganismPartCuratorAction() {
////			super("OrganismPartCurator");
////			putValue(Action.SHORT_DESCRIPTION,"");
//			putValue(Action.SHORT_DESCRIPTION,cr.getShortDescription());
//			putValue(Action.NAME,cr.getName() );
//		}
//
//		public void actionPerformed(ActionEvent e) {
//
//			final IEditPanelGUI editPanelView = new EditPanelGUIImpl();
//
//			curatorApp.getStatusBar().setProgressIndeterminate(true);
//			curatorApp.setMainPanel((JPanel) editPanelView, this);
//
//			final SwingWorker sw = new SwingWorker() {
//				@Override
//				public Object construct() {
////					ICurator cr = new OrganismPartCurator();
//					EditPanelControl ec = new EditPanelControl(editPanelView,cr, curatorApp);
//					ec.init();
//
//					return null;
//				}
//
//				@Override
//				public void finished() {
//					curatorApp.getStatusBar().setProgressIndeterminate(false);
//					curatorApp.getStatusBar().setStatus("Terms of Category 'OrganismPart' loaded.");
//				}
//			};
//			sw.start();
//
//		}
//
//	}	
//
	
	
}
