package de.ipk.pdwExpress.curator.tasks;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.control.ReplacementFactory;
import de.ipk.pdwExpress.curator.control.editpanel.EditPanelControl;
import de.ipk.pdwExpress.curator.images.AppImages;
import de.ipk.pdwExpress.curator.images.ImageHandler;
import de.ipk.pdwExpress.curator.util.CuratorToolKit;
import de.ipk.pdwExpress.curator.util.SwingWorker;
import de.ipk.pdwExpress.curator.view.panels.ReplacementFormDialog;
import de.ipk.pdwExpress.persistence.dataAccess.EditingSessionDAO;
import de.ipk.pdwExpress.persistence.dataAccess.ReplacementDAO;
import de.ipk.pdwExpress.persistence.dataAccess.TermDAO;
import de.ipk.pdwExpress.persistence.model.EditingSession;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Replacement;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmReplace;

class SaveReplacement extends AbstractAction {

	private static final long serialVersionUID = 1L;
	protected Logger logger;
	
	protected final CuratorApp curatorApp;
	
	public SaveReplacement(CuratorApp curatorApp){
		super("Save");
		this.curatorApp=curatorApp;
		logger = Logger.getLogger(this.getClass());
		putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.SAVE));
		putValue(Action.SHORT_DESCRIPTION,"Stores a Replacement in the database.");
	}


	public void actionPerformed(ActionEvent e) {
		logger.debug("editing finished.");
		
		
		final SwingWorker sw = new SwingWorker(){
			private ReplacementFormDialog rd;
			
			@Override
			public Object construct() {
				
//				ReplacementFactory.createNewReplacement();
				Replacement rpl = ReplacementFactory.getCurrentReplacement();
				EditingSession ese = ReplacementFactory.getCurrentEditingSession();
				
				// Dialog for Replacement
				rd = new ReplacementFormDialog(curatorApp.getJFrame());
				rd.setReplacement(rpl);							
				CuratorToolKit.centerFrame(rd);
				rd.setVisible(true);
				
				// try to save
				if (ReplacementFormDialog.CANCEL == rd.getPressedButton())
					return null;
				rpl=rd.getReplacement();
				
				ese.setEseDesc("Insert of new Terms");
				
				curatorApp.getStatusBar().setProgressIndeterminate(true);
				curatorApp.getStatusBar().setStatus("Saving replacement.");
				logger.debug("saving ...");

				List<TrmReplace> trmReplaces = new Vector<TrmReplace>(curatorApp.getCuratorActions().getEditControl().getTrmReplaces());

				List<Term> allTerms = CuratorToolKit.getTargetTermsList(trmReplaces);
				
				TermDAO termDao = new TermDAO();
				
				// find existing terms
				for (Term term : allTerms) {
					if (term.getTrmId() == null){
						Term newTerm=termDao.checkOnNaturalID(term, CuratorApp.getConnection());
						
						if (newTerm != null && newTerm.getTrmId() != null){
//							logger.info("oldTerm:"+term+"  newTerm:"+newTerm);
							trmReplaces=CuratorToolKit.replaceTargetTerms(trmReplaces, term, newTerm);
						}
					}
				}
				
				// make ExternDB equals
				allTerms = CuratorToolKit.getTargetTermsList(trmReplaces);
				for (Term term1 : allTerms) {
						for (Term term2 : allTerms) {
							ExternDB edb1 = term1.getExterndb();
							ExternDB edb2 = term2.getExterndb();
							if (edb1.equals(edb2) && edb1 != edb2){
//								logger.info("oldTerm:"+term1+"  newTerm:"+term2);
								trmReplaces=CuratorToolKit.replaceExternDB(trmReplaces, edb1, edb2);
							}
						}
				}
				
				// make  Terms equals
				allTerms = CuratorToolKit.getTargetTermsList(trmReplaces);
				for (Term term1 : allTerms) {
					for (Term term2 : allTerms) {
						if (term1.equals(term2) && term1!=term2){
//							logger.info("oldTerm:"+term1+"  newTerm:"+term2);
							trmReplaces=CuratorToolKit.replaceTargetTerms(trmReplaces, term1, term2);
						}
					}
				}	
				

				
				// Create association between TrmReplace and Replacement
				for ( TrmReplace replace : trmReplaces) {
					if (replace.getReplacement() == null){
						rpl.getTrmReplace().add(replace);
						replace.setReplacement(rpl);
					}
				}

				// Create association between new TargetTerms and EdittingSession
				for ( TrmReplace replace : trmReplaces) {
						List<Term> targets = replace.getDestinationTerms();
						for (Term term : targets) {
							System.out.println(term+" "+term.getEditingSession());
							if (term.getEditingSession() == null){
								term.setEditingSession(ese);
							}
						}
				}					
				
				
				// set replaces
				rpl.setTrmReplace(new HashSet(trmReplaces));
				
				//process saving
				ese.setEseUsername(rpl.getRplAuthor());
				EditingSessionDAO eseDAO = new EditingSessionDAO();
				eseDAO.save(ese, CuratorApp.getConnection());
				logger.info("EditingSession saved. "+ese);
				
				ese.setEseSuccessful('y');
				eseDAO.saveOrUpdate(ese, CuratorApp.getConnection());
				
				ReplacementDAO rplDao = new ReplacementDAO();
				rplDao.saveOrUpdate(rpl, CuratorApp.getConnection());
				logger.info("Replacement saved. "+rpl);

				// take care that new IDs are shown
				EditPanelControl ec = curatorApp.getCuratorActions().getEditControl();
				ec.clear();
				ec.addTrmReplaces(trmReplaces);
				
				return null;
			}

			@Override
			public void finished() {
				// TODO saved wird auch bei abbruch angezeigt

				curatorApp.getStatusBar().setProgressIndeterminate(false);
				curatorApp.getStatusBar().setStatus("Replacement saved.");
			}
		};
		sw.start();				
	}
	
	
	

}

