package de.ipk.pdwExpress.curator.tasks;

import java.awt.event.ActionEvent;
import java.sql.SQLException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.control.ReplacementFactory;
import de.ipk.pdwExpress.curator.control.overview.ReplacementOverviewControl;
import de.ipk.pdwExpress.curator.images.AppImages;
import de.ipk.pdwExpress.curator.images.ImageHandler;
import de.ipk.pdwExpress.curator.util.CuratorToolKit;
import de.ipk.pdwExpress.curator.util.SwingWorker;
import de.ipk.pdwExpress.curator.view.overview.ReplaceOverviewGUIImpl;
import de.ipk.pdwExpress.curator.view.panels.AboutDialog;

public class SysActions {

	protected final CuratorApp curatorApp;

	private Action exitAction;
	
//	private Action replacementOverViewAction;
	
	private Action saveReplacementAction;
	
	private Action newReplacementAction;

	private Action aboutAction;

	private Action replacementOverViewAction;

	public SysActions(CuratorApp app) {
		this.curatorApp = app;
	}

	public Action getExitAction() {

		if (exitAction == null) {
			exitAction = new AppExitAction();
		}

		return exitAction;
	}

	public Action getAboutAction() {
		if (aboutAction == null) {
			aboutAction = new AboutAction();
		}
		return aboutAction;
	}

	public Action getReplacementOverViewAction() {
		if (replacementOverViewAction == null) {
			replacementOverViewAction = new ReplacementOverviewAction();
		}
		return replacementOverViewAction;
	}
	
//	public Action getReplacementOverViewAction() {
//		if (replacementOverViewAction ==null){
//			replacementOverViewAction = new ReplacementOverviewAction();
//			replacementOverViewAction.setEnabled(false);
//		}
//		return replacementOverViewAction;
//	}

	public Action getSaveReplacementAction() {
		if (saveReplacementAction==null){
			saveReplacementAction=new SaveReplacement(curatorApp);
		}
		return saveReplacementAction;
	}	
	
	public Action getNewReplacementAction() {
		if (newReplacementAction == null){
			newReplacementAction = new NewReplacement();
		}
		return newReplacementAction;
	}		

	/**
	 * show About Dialog 
	 * @author Thomas
	 *
	 */
	class AboutAction extends AbstractAction {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public AboutAction() {
			super("About");
			putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.ABOUT));
			
		}

		public void actionPerformed(ActionEvent e) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					AboutDialog d = new AboutDialog(curatorApp.getJFrame());
					CuratorToolKit.centerFrame(d);
					d.setVisible(true);
				}
			});
		}
	}

	/**
	 * close Appliaction
	 * @author Thomas
	 *
	 */
	class AppExitAction extends AbstractAction {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public AppExitAction() {
			super("Exit");
			putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.EXIT));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				CuratorApp.getConnection().close();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			System.exit(0);
		}
	}

//	class ReplacementOverviewAction extends AbstractAction {
//		
//		/**
//		 * 
//		 */
//		private static final long serialVersionUID = 1L;
//
//		public ReplacementOverviewAction(){
//			super("Overview");
//			putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.OVERVIEW));
//		}
//
//		public void actionPerformed(ActionEvent e) {
//			
//			
//			
//
//		}
//
//	}

	
	class ReplacementOverviewAction extends AbstractAction {

		private static final long serialVersionUID = 1L;
		
		public ReplacementOverviewAction(){
			super("Overview");
			putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.OVERVIEW));
			putValue("LARGE_ICON",ImageHandler.getImageIcon(AppImages.OVERVIEW24));
			putValue(Action.SHORT_DESCRIPTION,"Showes a report on all replacements. Load, edit, execute replacements.");
		}

		public void actionPerformed(ActionEvent e) {
			curatorApp.getStatusBar().setProgressIndeterminate(true);
			curatorApp.getStatusBar().setStatus("Loading replacements.");
			
			ReplaceOverviewGUIImpl gui = new ReplaceOverviewGUIImpl();
			final ReplacementOverviewControl control = new ReplacementOverviewControl(gui,curatorApp);
			curatorApp.setMainPanel(gui,this);
			
			final SwingWorker sw = new SwingWorker(){
				@Override
				public Object construct() {
					control.init();					
					return null;
				}

				@Override
				public void finished() {
					curatorApp.getStatusBar().setProgressIndeterminate(false);
					curatorApp.getStatusBar().setStatus("Replacements loaded.");
				}
			};
			sw.start();
			

		}

	}
	
	class NewReplacement extends AbstractAction {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public NewReplacement(){
			super("New");
			putValue(Action.SMALL_ICON,ImageHandler.getImageIcon(AppImages.NEW));
			putValue(Action.SHORT_DESCRIPTION,"Discard current Replacement. Create new one.");			
		}

		public void actionPerformed(ActionEvent e) {
			ReplacementFactory.createNewReplacement();
			curatorApp.getCuratorActions().getEditControl().clear();
			curatorApp.getCuratorActions().getEmptyCuratorAction().actionPerformed(null);

		}

	}
	

	

}
