package de.ipk.pdwExpress.curator.util;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.sql.SQLException;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.ols.connection.IOlsDao;
import de.ipk.pdwExpress.ols.connection.OlsDaoImpl;
import de.ipk.pdwExpress.ols.model.OlsOntology;
import de.ipk.pdwExpress.ols.model.OlsTerm;
import de.ipk.pdwExpress.persistence.dataAccess.ExternDbDAO;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmCategoryTree;
import de.ipk.pdwExpress.persistence.model.TrmReplace;



public class CuratorToolKit {
	
	/**
	 * zentriert uebergebenes Fenster auf Bildschirm
	 * @param win
	 */
	public static void centerFrame(Window win){
		
		Dimension scrDim = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension winDim = win.getSize();
		win.setLocation(
			scrDim.width / 2 - winDim.width / 2,
			scrDim.height / 2 - winDim.height / 2);
		
	}//centerFrame
	
	public static Dimension getPercentScreenDim(double width, double height){
		Dimension scrDim = Toolkit.getDefaultToolkit().getScreenSize();
		int newWidth =(int)Math.round( width*scrDim.getWidth() );
		int newHeight = (int)Math.round( height*scrDim.getHeight());
		Dimension newDim = new Dimension(newWidth,newHeight);
		return newDim;
	}
	
	/**
	 * Wrappes all TrmCategories in a DefaultMutableTreeNode
	 * 
	 * @param root
	 * @return
	 */
	public static DefaultMutableTreeNode createTreeNodes(TrmCategoryTree root) {
		
//		if (rootNode == null){
			DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(root);
			addChilds(rootNode,root.getTrmCategoryChilds());
//		}
		
		return rootNode;
	}
	
	private static void addChilds(DefaultMutableTreeNode rootNode,Collection<TrmCategoryTree> catRoot){
		for (TrmCategoryTree cat: catRoot){
			DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(cat);
			rootNode.add(newNode);
			addChilds(newNode,cat.getTrmCategoryChilds());
		}			
	}
	
	/**
	 * Converts a Ols Term into a Term of pdw_stage
	 * 
	 * @param olsTerm
	 * @return
	 */
	public static Term olsTermToStageTerm(OlsTerm olsTerm){
		Term term = new Term();
		term.setTrmValue(olsTerm.getTerm_name());
		term.setTrmDesc(olsTerm.getDefinition());
		term.setTrmEdbacc(olsTerm.getIdentifier());
		
		// TODO  category
		ExternDbDAO edbDao = new ExternDbDAO();
		OlsOntology olsOntology=null;
		try {
			IOlsDao olsDao = CuratorApp.getOlsDao();
			olsOntology = olsDao.getOntologyByID(olsTerm.getOntologyID());
		} catch (SQLException e) {
			return null;
		}
		
		String shortName = olsOntology.getShortName();
		ExternDB edb = edbDao.getByShortName(shortName, CuratorApp.getConnection());
		
		term.setExterndb(edb);
		term.setTrmCategory(null);
		return term;
	}

//	public static List<TrmReplace> copyTrl(List<TrmReplace> trmReplaces) {
//		List<TrmReplace> newReplaces = new Vector<TrmReplace>();
//		
//		for (TrmReplace replace : trmReplaces) {
////			TrmReplace newReplace=replace;
//			TrmReplace newReplace = new TrmReplace();
//			newReplace.setTrlId(replace.getTrlId());
//			newReplace.setType(replace.getType());
//			newReplace.setSourceTerm(replace.getSourceTerm());
//			
//			List<Term> tagets = replace.getDestinationTerms();
//			for (Term taget : tagets) {
//				newReplace.getDestinationTerms().add(taget);
//			}
//			
//			
//			newReplaces.add(newReplace);
//		}
//		
//		return newReplaces;
//	}
	
	
	/**
	 * 
	 * Replaces targetTerms
	 * 
	 * compares using == not equals()
	 * @param oldTerm The Old Term
	 * @param newTerm The new Term
	 * @return
	 */
	public static List<TrmReplace> replaceTargetTerms(List<TrmReplace> trmReplaces, Term oldTerm, Term newTerm) {
		
		for ( TrmReplace replace : trmReplaces) {
			
			List<Term> targets = replace.getDestinationTerms();
			List<Term> newTargets = new Vector<Term>(targets);
			
			for (Term term : targets) {
				if (oldTerm==term){
						newTargets.remove(term);
						newTargets.add(newTerm);
				}
			}
			replace.setDestinationTerms(newTargets);
			
		}		
		return trmReplaces;
		
	}
	
	/**
	 * Creates a new List, which contains all Destination terms
	 * 
	 * @param trmReplaces
	 * @return
	 */
	public static List<Term> getTargetTermsList(List<TrmReplace> trmReplaces){
		List<Term> allTerms = new Vector<Term>();
		for ( TrmReplace replace : trmReplaces) 
			for (Term term : replace.getDestinationTerms()) 
				allTerms.add(term);		
		
		return allTerms;
	}

	/**
	 * replaces all referenced ExternDB in the list
	 * 
	 * @param trmReplaces List of TrmReplace, which contains terms
	 * @param oldEdb Old ExternDB
	 * @param newEdb New Extern DB
	 * @return
	 */
	public static List<TrmReplace> replaceExternDB(List<TrmReplace> trmReplaces, ExternDB oldEdb, ExternDB newEdb) {

		for (TrmReplace replace : trmReplaces) {
			for(Term term : replace.getDestinationTerms()){
				if(term.getExterndb() == oldEdb)
					term.setExterndb(newEdb);
			}
		}
		return trmReplaces;

	}
	


}

























