package de.ipk.pdwExpress.curator.view.editors;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.Collection;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellEditor;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.CuratorApp;
import de.ipk.pdwExpress.curator.util.CuratorToolKit;
import de.ipk.pdwExpress.curator.view.selecttree.CategoryTreePanelImpl;
import de.ipk.pdwExpress.curator.view.selecttree.ITreePanel;
import de.ipk.pdwExpress.curator.view.selecttree.nodeRenderer.CategoryTermNodeDetailsRenderer;
import de.ipk.pdwExpress.ols.connection.IOlsDao;
import de.ipk.pdwExpress.ols.model.OlsTerm;
import de.ipk.pdwExpress.ols.view.OlsTreePanel;
import de.ipk.pdwExpress.persistence.model.Term;
import de.ipk.pdwExpress.persistence.model.TrmCategoryTree;

/**
 * TableCellEditor to select a term
 * 
 * @author Thomas Rutkowski
 *
 */
public class SelectTermTableCellEditor extends AbstractCellEditor implements TableCellEditor, ActionListener {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected Logger logger;
//	private  TrmCategory currentCategory ;
	private  TrmCategoryTree rootCategory;

	private JButton button;

	private JDialog dialog;
	
	private JFrame parentFrame;
	
	private OlsTreePanel olsPanel ;
	
//	private  TrmCategory currentCategory;
	
//	private DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Empty");
	private CategoryTreePanelImpl treePanel;
	
	private Term selectedTerm;
	

	protected static final String EDIT = "edit";

	public SelectTermTableCellEditor(){
		
		logger=Logger.getLogger(this.getClass());
		
		button = new JButton();
		button.setActionCommand(EDIT);
		button.addActionListener(this);
		button.setBorderPainted(false);		
	}
	
	public SelectTermTableCellEditor(TrmCategoryTree rootCategory) {
		this();
		this.rootCategory=rootCategory;
	}
	
	private JDialog getDialog(){
		
		if(dialog==null){
			dialog=new JDialog(parentFrame,true);
			//Set up the dialog that the button brings up.
			dialog.setSize(460,490);
			dialog.setSize(CuratorToolKit.getPercentScreenDim(0.6, 0.6));
			dialog.getContentPane().setLayout(new GridLayout(1,1));

			CategoryTreePanelImpl tp = getTreePanel();
			tp.setRoot(getRootTreeNode());

			
			getOntologyPanel().addActionListener(this);
			JTabbedPane tabbedPane = new JTabbedPane();
			tabbedPane.addTab("Term from stage", tp);
			tabbedPane.addTab("Term from OLS", getOntologyPanel());
			
			
			CuratorToolKit.centerFrame(dialog);
			dialog.getContentPane().add(tabbedPane);
		}
		
		return dialog;
	}
	
	private OlsTreePanel getOntologyPanel(){
		if (olsPanel == null){
		IOlsDao dao;
		try {
			dao = CuratorApp.getOlsDao();
		} catch (SQLException e) {
			logger.error(e);
			e.printStackTrace();
			return null;				
		}
		olsPanel = new OlsTreePanel(dao);		
		}
		return olsPanel;
		
	}
	
	private CategoryTreePanelImpl getTreePanel(){
		
		if (treePanel ==null){
			treePanel = new CategoryTreePanelImpl(this);
//			treePanel.setNodeDetailsRenderer(new CategoryNodeDetailsRenderer());
			treePanel.setNodeDetailsRenderer(new CategoryTermNodeDetailsRenderer());
			treePanel.setRoot(getRootTreeNode());
			treePanel.setDetailSelectAble(true);
		}
		return treePanel;
	}

	private DefaultMutableTreeNode getRootTreeNode() {
		
//		if (rootNode == null){
			DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(rootCategory);
			addChilds(rootNode,rootCategory.getTrmCategoryChilds());
//		}
		
		return rootNode;
	}
	
	private void addChilds(DefaultMutableTreeNode rootNode,Collection<TrmCategoryTree> catRoot){
		for (TrmCategoryTree cat: catRoot){
			DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(cat);
			rootNode.add(newNode);
			addChilds(newNode,cat.getTrmCategoryChilds());
		}			
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {

		if (EDIT.equals(e.getActionCommand())) {
			logger.debug("Edit button pressed.");
			//The user has clicked the cell, so bring up the dialog.
			getDialog().setVisible(true);
			fireEditingStopped(); //Make the renderer reappear.			
		} 
		
		else if (ITreePanel.CANCEL.equals(e.getActionCommand()) || OlsTreePanel.ACT_CMD_CANCEL.equals(e.getActionCommand())){ 
			logger.debug("Selection Canceled.");
			selectedTerm=null;
			
		}else if (ITreePanel.OK.equals(e.getActionCommand())){ 
			logger.debug("Selection Ok. (StageTerm)");
			selectedTerm=(Term)getTreePanel().getSelectedDetail();
			
		}else if (OlsTreePanel.ACT_CMD_OK.equals(e.getActionCommand())){ 
			logger.debug("Selection Ok. (OntologyTerm)");
			OlsTerm olsTerm = getOntologyPanel().getSelectedTerm();
			selectedTerm=CuratorToolKit.olsTermToStageTerm(olsTerm);
			
		}else{
			logger.warn("Unsupported action. action:"+e);
			return;
		}
		
		logger.debug("Selected Term: "+selectedTerm);
		dialog.setVisible(false);

	}

	/**
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 * @return Returns the select term; NULL if nothing is selected.
	 */
	public Object getCellEditorValue() {
		logger.debug("getCellEditorValue() selectedTerm:"+selectedTerm);
		return selectedTerm;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {

		parentFrame = (JFrame) SwingUtilities.getRoot(table);		
	
//		currentCategory = (TrmCategory) value;
//		String text = (currentCategory==null ? "":currentCategory.getTctName());
//		button.setText(text);
		button.setText(value==null ? "" : value.toString());
		
		return button;
	}

//	public void setCurrentCategory(TrmCategory currentCategory) {
//		this.currentCategory = currentCategory;
//	}

//	public TrmCategoryTree getRootCategory() {
//		return rootCategory;
//	}

	
	public void setRootCategory(TrmCategoryTree rootCategory) {
		this.rootCategory = rootCategory;
//		this.rootNode=null;
	}


}
