package de.ipk.pdwExpress.curator.view.editors;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellEditor;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.util.CuratorToolKit;
import de.ipk.pdwExpress.curator.view.selecttree.CategoryTreePanelImpl;
import de.ipk.pdwExpress.curator.view.selecttree.ITreePanel;
import de.ipk.pdwExpress.curator.view.selecttree.nodeRenderer.CategoryNodeDetailsRenderer;
import de.ipk.pdwExpress.persistence.model.TrmCategory;
import de.ipk.pdwExpress.persistence.model.TrmCategoryTree;

/**
 * TableCellEditor to select a category
 * 
 * @author Thomas Rutkowski
 *
 */
public class TrmCategoryEditor extends AbstractCellEditor implements TableCellEditor,
		ActionListener {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected Logger logger;
	private  TrmCategory currentCategory ;
	private  TrmCategoryTree rootCategory;

	private JButton button;

	private JDialog dialog;
	
	private JFrame parentFrame;
	
	private DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Empty");
	private CategoryTreePanelImpl treePanel;
	
	

	protected static final String EDIT = "edit";

	public TrmCategoryEditor(){
		
		logger=Logger.getLogger(this.getClass());
		
		button = new JButton();
		button.setActionCommand(EDIT);
		button.addActionListener(this);
		button.setBorderPainted(false);		
	}
	
	public TrmCategoryEditor(TrmCategoryTree rootCategory) {
		this();
		this.rootCategory=rootCategory;
	}
	
	private JDialog getDialog(){
		
		if(dialog==null){
			dialog=new JDialog(parentFrame,true);
			//Set up the dialog that the button brings up.
			dialog.setSize(CuratorToolKit.getPercentScreenDim(0.6, 0.6));		
			dialog.getContentPane().setLayout(new GridLayout(1,1));
			CategoryTreePanelImpl tp = getTreePanel();
			tp.setRoot(CuratorToolKit.createTreeNodes(this.rootCategory));
			CuratorToolKit.centerFrame(dialog);
			dialog.getContentPane().add(tp);
		}
		
		return dialog;
	}
	
	private CategoryTreePanelImpl getTreePanel(){
		
		if (treePanel ==null){
			treePanel = new CategoryTreePanelImpl(this);
			treePanel.setNodeDetailsRenderer(new CategoryNodeDetailsRenderer());
//			treePanel.setNodeDetailsRenderer(new CategoryTermNodeDetailsRenderer());			
			treePanel.setRoot(CuratorToolKit.createTreeNodes(this.rootCategory));
		}
		return treePanel;
	}



	public void actionPerformed(ActionEvent e) {

		if (EDIT.equals(e.getActionCommand())) {
			//The user has clicked the cell, so
			//bring up the dialog.
			getDialog().setVisible(true);
			fireEditingStopped(); //Make the renderer reappear.
			
		} else if (ITreePanel.CANCEL.equals(e.getActionCommand())){ 
			logger.debug("Selection Canceled.");
//			currentCategory = null;
		}else if (ITreePanel.OK.equals(e.getActionCommand())){ 
			logger.debug("Selection Ok.");
			currentCategory = (TrmCategory) getTreePanel().getSelectedEntries().get(0);
		}
		
		logger.debug("Selected Category: "+currentCategory);
		dialog.setVisible(false);

	}

	//Implement the one CellEditor method that AbstractCellEditor doesn't.
	public Object getCellEditorValue() {
		return currentCategory;
	}

	//Implement the one method defined by TableCellEditor.
	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		
		parentFrame = (JFrame) SwingUtilities.getRoot(table);		
	
		currentCategory = (TrmCategory) value;
		String text = (currentCategory==null ? "":currentCategory.getTctName());
		button.setText(text);
		
		return button;
	}

	public void setCurrentCategory(TrmCategory currentCategory) {
		this.currentCategory = currentCategory;
	}

	public TrmCategoryTree getRootCategory() {
		return rootCategory;
	}

	public void setRootCategory(TrmCategoryTree rootCategory) {
		this.rootCategory = rootCategory;
		this.rootNode=null;
	}


}
