package de.ipk.pdwExpress.curator.view.selecttree;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.tree.TreeNode;

import org.apache.log4j.Logger;

import de.ipk.pdwExpress.curator.view.selecttree.nodeRenderer.CategoryTermNodeDetailsRenderer;
import de.ipk.pdwExpress.persistence.model.Term;

/**
 * Dialog with Tree (Categories) and Table (Terms). <br>
 * 
 * Allows to Select a Term.
 * 
 * @author Thomas Rutkowski
 *
 */
public class TermTreeDialog extends JDialog implements ActionListener{

	protected Logger logger;
	private static final long serialVersionUID = 1L;

	private JPanel jContentPane = null;

	private ITreePanel treePanel = null;  //  @jve:decl-index=0:
	private final TreeNode root;
	private Term selectedTerm;

	private TermTreeDialog(){
		this(null,null);
		setVisible(true);
	}
	
	public TermTreeDialog(JFrame owner, TreeNode root) {
		super(owner,true);
		logger=Logger.getLogger(this.getClass());
		this.root=root;
		initialize();
	}


	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(800, 450);
		this.setTitle("Select Term");
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add((JPanel)getTreePanel(), BorderLayout.CENTER);
		}
		return jContentPane;
	}


	/**
	 * This method initializes treePanel	
	 * 	
	 * @return de.ipk.pdwExpress.curator.view.selecttree.TreePanel	
	 */
	private ITreePanel getTreePanel() {
		if (treePanel == null) {
			treePanel = new CategoryTreePanelImpl(this);
			treePanel.setNodeDetailsRenderer(new CategoryTermNodeDetailsRenderer());
			treePanel.setRoot(root);
			treePanel.setDetailSelectAble(true);
		}
		return treePanel;
	}

	
	
	public Term getSelectedEntry() {
		return selectedTerm;
	}

	public void actionPerformed(ActionEvent e) {
		
		if (e.getActionCommand() == ITreePanel.OK){
			int index = treePanel.getSelectedDetailIndex();
			logger.debug("selected detailIndex:"+index);
			selectedTerm=(Term)treePanel.getSelectedDetail();
		}
		
		else if (e.getActionCommand() == ITreePanel.CANCEL){
			logger.debug("Selection canceled.");
//			selectedTerm=(Term)treePanel.getSelectedDetail();
			selectedTerm=null;
		}else{
			logger.warn("Unsupported action. action:"+e);
			return;
		}
		
		this.dispose();
		
	}



}  //  @jve:decl-index=0:visual-constraint="10,10"
