package de.ipk.pdwExpress.curator.view.viewPanel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;

import org.apache.log4j.Logger;


import de.ipk.pdwExpress.curator.control.viewPanel.TermTableModel;
import de.ipk.pdwExpress.curator.control.viewPanel.TrmReplaceTableModel;
import de.ipk.pdwExpress.curator.view.renderer.ExternDBRenderer;
import de.ipk.pdwExpress.curator.view.renderer.TrlTypeTableCellRenderer;
import de.ipk.pdwExpress.curator.view.renderer.TrmCategoryRenderer;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.TrlType;
import de.ipk.pdwExpress.persistence.model.TrmCategory;

/**
 * Shows Source terms and for each selected source term the target terms. 
 *  
 * @author Thomas Rutkowski
 *
 */
public class ViewPanelGUIImpl extends JPanel implements IViewPanelGUI {

	private static final long serialVersionUID = 1L;
	protected Logger logger;

	protected final TableCellRenderer trmCategoryRenderer= new TrmCategoryRenderer();
	protected final TableCellRenderer externDBRenderer =new ExternDBRenderer();  //  @jve:decl-index=0:
	protected final TableCellRenderer trlTypeRenderer =new TrlTypeTableCellRenderer(); 		
	
	
	protected JLabel lblSource = null;
	protected JLabel lblDest = null;

	private JScrollPane scrollDest = null;
	private JTable tableDestination = null;
	private JScrollPane scrollSource = null;
	private JTable tableSource = null;

	/**
	 * This is the default constructor
	 */
	public ViewPanelGUIImpl() {
		super();
		logger=Logger.getLogger(this.getClass());
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
		gridBagConstraints21.gridx = 1;
		gridBagConstraints21.insets = new Insets(6, 6, 0, 0);
		gridBagConstraints21.gridy = 2;
		GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
		gridBagConstraints11.gridx = 0;
		gridBagConstraints11.insets = new Insets(6, 6, 0, 0);
		gridBagConstraints11.anchor = GridBagConstraints.EAST;
		gridBagConstraints11.gridy = 2;
		GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
		gridBagConstraints8.gridx = 3;
		gridBagConstraints8.insets = new Insets(12, 0, 0, 0);
		gridBagConstraints8.anchor = GridBagConstraints.SOUTHEAST;
		gridBagConstraints8.gridy = 3;
		GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
		gridBagConstraints4.gridx = 3;
		gridBagConstraints4.anchor = GridBagConstraints.EAST;
		gridBagConstraints4.insets = new Insets(6, 6, 0, 0);
		gridBagConstraints4.gridy = 2;
		gridBagConstraints4.weightx = 0;
		GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
		gridBagConstraints31.gridx = 2;
		gridBagConstraints31.anchor = GridBagConstraints.EAST;
		gridBagConstraints31.insets = new Insets(6, 6, 0, 0);
		gridBagConstraints31.gridy = 2;
		GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
		gridBagConstraints3.fill = GridBagConstraints.BOTH;
		gridBagConstraints3.gridy = 1;
		gridBagConstraints3.weightx = 50;
		gridBagConstraints3.weighty = 1.0;
		gridBagConstraints3.gridx = 0;
		gridBagConstraints3.gridheight=1;
		gridBagConstraints3.gridwidth=2;
		gridBagConstraints3.insets = new Insets(0, 10, 0, 0);
		GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
		gridBagConstraints2.fill = GridBagConstraints.BOTH;
		gridBagConstraints2.gridy = 1;
		gridBagConstraints2.weightx = 50;
		gridBagConstraints2.weighty = 1.0;
		gridBagConstraints2.gridx = 2;
		gridBagConstraints2.gridwidth=2;
		gridBagConstraints2.insets = new Insets(0, 10, 0, 0);
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.anchor = GridBagConstraints.WEST;
		gridBagConstraints1.insets = new Insets(0, 10, 6, 0);
		gridBagConstraints1.gridx = 2;
		gridBagConstraints1.weightx = 50;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridy = 0;
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.insets = new Insets(0, 10, 6, 0);
		gridBagConstraints.gridx = 0;
		gridBagConstraints.weightx = 50;
		lblDest = new JLabel();
		lblDest.setText("Target(s)");
		lblSource = new JLabel();
		lblSource.setText("Source Terms");
		this.setLayout(new GridBagLayout());
		this.setSize(504, 330);
		this.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
		this.add(lblSource, gridBagConstraints);
		this.add(lblDest, gridBagConstraints1);
		this.add(getScrollDest(), gridBagConstraints2);
		this.add(getScrollSource(), gridBagConstraints3);

	}
	
	public void setSourceTableModel(TrmReplaceTableModel model) {
		getTableSource().setModel(model);				
	}
	
	public void setDestinationTableModel(TermTableModel model) {
		getTableDestination().setModel(model);
//		tableModelDestination=model;		
	}
	
	/**
	 * This method initializes scrollDest	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	protected JScrollPane getScrollDest() {
		if (scrollDest == null) {
			scrollDest = new JScrollPane();
			scrollDest.setViewportView(getTableDestination());
		}
		return scrollDest;
	}

	/**
	 * This method initializes tableDestination	
	 * 	
	 * @return javax.swing.JTable	
	 */
	protected JTable getTableDestination() {
		if (tableDestination == null) {
			tableDestination = new JTable(new TermTableModel());
			tableDestination.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

//			((JideTable)tableDestination).setColumnAutoResizable(true);
			tableDestination.setDefaultRenderer(ExternDB.class, externDBRenderer);
			tableDestination.setDefaultRenderer(TrmCategory.class, trmCategoryRenderer);
		}
		return tableDestination;
	}

	/**
	 * This method initializes scrollSource	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	protected JScrollPane getScrollSource() {
		if (scrollSource == null) {
			scrollSource = new JScrollPane();
			scrollSource.setViewportView(getTableSource());
			
		}
		return scrollSource;
	}

	/**
	 * This method initializes tableSource	
	 * 	
	 * @return javax.swing.JTable	
	 */
	protected JTable getTableSource() {
		if (tableSource == null) {
			tableSource = new JTable(new TermTableModel());
			tableSource.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
//			((JideTable)tableSource).setColumnAutoResizable(true);		
			tableSource.setDefaultRenderer(ExternDB.class, externDBRenderer);
			tableSource.setDefaultRenderer(TrlType.class, trlTypeRenderer);
			tableSource.setDefaultRenderer(TrmCategory.class, trmCategoryRenderer);
			
//			tableSource.getSelectionModel().addListSelectionListener(this);			
		}
		return tableSource;
	}

	public void addDestinationTableSelectListener(ListSelectionListener l) {
		getTableDestination().getSelectionModel().addListSelectionListener(l);
	}


	public void addSourceTableSelectListener(ListSelectionListener l) {
		getTableSource().getSelectionModel().addListSelectionListener(l);
		
		getTableSource().getSelectionModel().
		addListSelectionListener(new ListSelectionListener(){

			public void valueChanged(ListSelectionEvent e) {
				getTableSource().getSelectedRow();
				
			}
		
		});
	}

	public int getSelectSourceIndex() {
		return getTableSource().getSelectedRow();
	}


	public int getSelectedDestinationIndex() {
		return getTableDestination().getSelectedRow();
	}

	public void setSelectedSourceIndex(int i) {
		
		if(getTableSource().getRowCount() > i)
			getTableSource().getSelectionModel().setSelectionInterval(i,i);
		else
			if(getTableSource().getRowCount() > 0)
				getTableSource().getSelectionModel().setSelectionInterval(0,0);
			
	}

	

}
