package de.ipk.pdwExpress.ols.connection;

import java.sql.SQLException;
import java.util.List;

import javax.swing.tree.TreeNode;

import de.ipk.pdwExpress.ols.model.OlsOntology;
import de.ipk.pdwExpress.ols.model.OntologyRoot;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;

/**
 * Provides access to the Ontology LookUp Service.
 * 
 * @author Thomas Rutkowski
 *
 */
public interface IOlsDao {
	

	/**
	 * closes intern Preparedstatements
	 *
	 */
	public void close();
	/**
	 * Search for an ontology with the given shortname.
	 * 
	 * @param shortName
	 * @return NULL if nothing was found.
	 * @throws SQLException
	 */
	public ExternDB getExternDBByShortName(String shortName) throws SQLException;
	
	/**
	 * Query Term table from ols.
	 * Search for a Term with the given accession. <br><br>
	 * 
	 * TODO: Handel diverent version of a Ontology
	 * 
	 * @param acc A accession. E.g. 'GO:0001234'
	 * @return NULL if tothing was found.
	 * @throws SQLException
	 */
	public Term getTermByAccession(String acc) throws SQLException;
	
	/**
	 * Query Term table from ols.
	 * Executes query with <code>like</code> expression.
	 * 
	 * @param value Like Expression. 
	 * @return List of all terms matching the expression. Never Null.
	 */	
	public List<Term> getTermsByValue(String term_name) throws SQLException;
	
	/**
	 * 
	 * @param ontologyID
	 * @return The rootTerm of the ontology. UserObject is of type OLSTerm 
	 * @throws SQLException 
	 */
	public TreeNode getOntologyRootNode(Integer ontologyID) throws SQLException;
	
	/**
	 * List of the newest fully loaded ontologies.
	 * 
	 * @return
	 */
	public List<OlsOntology> getOntologies()throws SQLException;
	
	/**
	 * 
	 * @return Root terms of ontology
	 */
	public List<OntologyRoot> getOntologyRoots()throws SQLException;
	
	/**
	 * Search for an Ontolgy with the given ID
	 * 
	 * @param ontologyID
	 * @return The Ontology or NULL if nothing was found.
	 */
	public OlsOntology getOntologyByID(Integer ontologyID) throws SQLException;
	
}
