package de.ipk.pdwExpress.ols.connection;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;

import de.ipk.pdwExpress.ols.model.OlsTerm;
import de.ipk.pdwExpress.ols.model.OlsOntology;
import de.ipk.pdwExpress.ols.model.OntologyRoot;
import de.ipk.pdwExpress.persistence.model.ExternDB;
import de.ipk.pdwExpress.persistence.model.Term;

/**
 * 
 *  
 * @author Thomas Rutkowski
 *
 */
public class OlsDaoImpl implements IOlsDao {
	
	protected Logger logger;
	

	
	/**
	 * A Connection which has permissions to read <code>OLD_DB_USER</code>
	 */
	private final Connection connection;
	
	private PreparedStatement stmtEDBByName;
	private PreparedStatement stmtTermByAcc;
	private PreparedStatement stmtTermsByVal;
	private PreparedStatement stmtEDBByOlsId;
	private PreparedStatement stmtTree;
	private PreparedStatement stmtNode;
	private PreparedStatement stmtOntologies;	
	private PreparedStatement stmtOntologyByID;
	private PreparedStatement stmtRootTermByOnt;
	
	
	public OlsDaoImpl(Connection connection) throws SQLException {
		logger = Logger.getLogger(this.getClass());
		this.connection = connection;
		initStatements();
	}

	
	private void initStatements() throws SQLException {
		logger.debug("Preparing statements.");
		
		stmtEDBByName = getConnection().prepareStatement("select * from "+IOlsQueries.OLS_DB_USER+".new_ontology where shortname = ?");
		stmtTermByAcc = getConnection().prepareStatement("select * from "+IOlsQueries.OLS_DB_USER+".term where identifier = ?");
		stmtTermsByVal = getConnection().prepareStatement("select * from "+IOlsQueries.OLS_DB_USER+".term where term_name like(?)");
		stmtEDBByOlsId = getConnection().prepareStatement("select * from "+IOlsQueries.OLS_DB_USER+".ontology where ontology_id = ?");
		stmtTree = getConnection().prepareStatement(IOlsQueries.TREEQUERY);
		stmtTree.setFetchSize(200);
		stmtNode = getConnection().prepareStatement(IOlsQueries.NODE);
		stmtOntologies= getConnection().prepareStatement(IOlsQueries.ONTOLOGIES);
		stmtRootTermByOnt= getConnection().prepareStatement(IOlsQueries.ROOTTERM);
		stmtOntologyByID= getConnection().prepareStatement(IOlsQueries.ONTOLOGYBYID);
		
		logger.debug("Statements prepared.");
	}



	/**
	 * 
	 * @param id
	 * @return
	 * @throws SQLException
	 */
	private ExternDB getExternDbByOlsId(int id) throws SQLException {
		
		PreparedStatement stmt =getStmtEDBByOlsId();
		stmt.setInt(1, id);
		
		ResultSet rs = stmt.executeQuery();
		
		ExternDB edb =null;
		if (rs.next()){
			edb = new ExternDB();
			edb.setEdbName(rs.getString("FULLNAME"));
			edb.setEdbDesc(rs.getString("DEFINITION"));
			edb.setEdbUrl(rs.getString("QUERY_URL"));
			edb.setEdbShortName(rs.getString("SHORTNAME"));
		}
		DbUtils.closeQuietly(rs);
		 
		return edb;
	}

		
	protected final Connection getConnection() {
		return connection;
	}

	
	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#getExternDBByShortName(java.lang.String)
	 */
	public ExternDB getExternDBByShortName(String shortName) throws SQLException {
		PreparedStatement stmt =getStmtEDBByName();
		stmt.setString(1, shortName);
		
		ResultSet rs = stmt.executeQuery();
		
		ExternDB edb =null;
		if (rs.next()){
			edb = new ExternDB();
			edb.setEdbName(rs.getString("FULLNAME"));
			edb.setEdbDesc(rs.getString("DEFINITION"));
			edb.setEdbUrl(rs.getString("QUERY_URL"));
		}
		DbUtils.closeQuietly(rs);
		 
		return edb;
	}

	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#getTermByAccession(java.lang.String)
	 */
	public Term getTermByAccession(String acc) throws SQLException {
		
		PreparedStatement stmt =getStmtTermByAcc();
		stmt.setString(1, acc);
		
		ResultSet rs = stmt.executeQuery();
		
		Term term =null;
		if (rs.next()){
			term = new Term();
			term.setTrmValue(rs.getString("TERM_NAME"));
			term.setTrmDesc(rs.getString("DEFINITION"));
			term.setTrmImportdate(new Date());
			term.setTrmEdbacc(rs.getString("IDENTIFIER"));
			term.setExterndb(getExternDbByOlsId(rs.getInt("ONTOLOGY_ID")));
		}
		DbUtils.closeQuietly(rs);
		 
		return term;
	}
	

	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#getTermsByValue(java.lang.String)
	 */
	public List<Term> getTermsByValue(String term_name) throws SQLException {
		PreparedStatement stmt = getStmtTermsByVal();
		stmt.setString(1, term_name);
		
		List<Term> terms = new Vector<Term>();
		ResultSet rs = stmt.executeQuery();
	
		while (rs.next()) {
			Term term = new Term();
			term.setTrmValue(rs.getString("TERM_NAME"));
			term.setTrmDesc(rs.getString("DEFINITION"));
			term.setTrmImportdate(new Date());
			term.setTrmEdbacc(rs.getString("IDENTIFIER"));
			term.setExterndb(getExternDbByOlsId(rs.getInt("ONTOLOGY_ID")));

			terms.add(term);
		}
		DbUtils.closeQuietly(rs);
		
		return terms;
	}

	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#getOntologyRoot(java.lang.Integer)
	 */
	public TreeNode getOntologyRootNode(Integer rootID) throws SQLException {
		
		logger.debug("loading ontology with root id: "+rootID);
		
		ResultSet rs =null;
		DefaultMutableTreeNode rootNode =null;
		try {
			
			OlsTerm rootTerm = getTerm(rootID);
			
			if (rootTerm == null){
				logger.warn("Term not found. term_id:"+rootID);
				return null;
			}
			rootNode = new DefaultMutableTreeNode(rootTerm);
			
			logger.debug("loading ontology  OntologyID:"+rootTerm.getOntologyID());
			
			PreparedStatement stmt =getStmtTree();
			stmt.setInt(1, rootTerm.getOntologyID());
			stmt.setInt(2, rootTerm.getOntologyID());
			stmt.setInt(3, rootID);
			rs = stmt.executeQuery();
	
			if (rs.next())
				addSubTree(rootNode , rs);
				
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("Error while loading Ontology"+e);
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		logger.debug("Ontologies loaded. root:"+rootNode);
		return rootNode;
	}	

	


	/**
	 * 
	 * @param nodeID TERM_ID of the wanted term
	 * @return NULL if nothing was found.
	 * @throws SQLException
	 */
	private OlsTerm getTerm(Integer nodeID) throws SQLException {
		
		ResultSet rs =null;
		PreparedStatement stmt = getStmtNode();
		
		try {
			stmt.setInt(1, nodeID);
			rs = stmt.executeQuery();
			
			if (rs.next())
				return getTerm(rs);
		} catch (SQLException e) {
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		return null;
	}

	/**
	 * 
	 * @param father FatherNode
	 * @param rs The ResultSet of connect by prior
	 * @return FALSE if there are no more rows
	 * @throws SQLException
	 */
	protected boolean addSubTree(DefaultMutableTreeNode father, ResultSet rs) throws SQLException{

		final int startLevel = rs.getInt("LEVEL");
		boolean moreChilds = false;
		
		do{
			int currLevel=rs.getInt("LEVEL");
			
			DefaultMutableTreeNode child = new DefaultMutableTreeNode(getTerm(rs));
			father.add(child);	
			
			if(!rs.next())
				return false;
			
			if (rs.getInt("LEVEL") > currLevel){
				// es geht ein level tiefer
				if(!addSubTree(child,rs))
					return false;
			}
			currLevel=rs.getInt("LEVEL");
			moreChilds = startLevel==currLevel;
				
		}while(moreChilds);
		
		return true;

	}//addSubTree	
	
	

	private OlsTerm getTerm(ResultSet rs) throws SQLException {
		OlsTerm term = new OlsTerm();
		term.setDefinition(rs.getString("DEFINITION"));
		term.setIdentifier(rs.getString("IDENTIFIER"));
		term.setLeaf(rs.getBoolean("IS_LEAF"));
		term.setNameSpace(rs.getString("NAMESPACE"));
		term.setRootTerm(rs.getBoolean("IS_ROOT_TERM"));
		term.setTerm_name(rs.getString("TERM_NAME"));
		term.setObsolete(rs.getBoolean("IS_OBSOLETE"));
		term.setTermID(rs.getInt("TERM_ID"));
		term.setOntologyID(rs.getInt("ONTOLOGY_ID"));
		
		return term;
	}

	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#close()
	 */
	public void close() {
		DbUtils.closeQuietly(stmtEDBByName);
		DbUtils.closeQuietly(stmtTermByAcc);
		DbUtils.closeQuietly(stmtTermsByVal);
		DbUtils.closeQuietly(stmtEDBByOlsId);
		DbUtils.closeQuietly(stmtTree);
		DbUtils.closeQuietly(stmtNode);
		DbUtils.closeQuietly(stmtOntologies);	
		DbUtils.closeQuietly(stmtOntologyByID);		
	}

	protected final PreparedStatement getStmtEDBByName() {
		return stmtEDBByName;
	}

	protected final PreparedStatement getStmtEDBByOlsId() {
		return stmtEDBByOlsId;
	}

	protected final PreparedStatement getStmtNode() {
		return stmtNode;
	}

	protected final PreparedStatement getStmtTermByAcc() {
		return stmtTermByAcc;
	}

	protected final PreparedStatement getStmtTermsByVal() {
		return stmtTermsByVal;
	}

	protected final PreparedStatement getStmtTree() {
		return stmtTree;
	}


	/*
	 * (non-Javadoc)
	 * @see de.ipk.pdwExpress.ols.connection.IOlsDao#getOntologies()
	 */
	public List<OlsOntology> getOntologies() throws SQLException {
		
		logger.debug("Loading Ontologies.");
		
		Vector<OlsOntology> ontologies = new Vector<OlsOntology>();
		ResultSet rs=null;
		try {
			rs = getStmtOntologies().executeQuery();
			while (rs.next()){
				ontologies.add(getOntology(rs));
			}
		} catch (SQLException e) {
			logger.error("Error while loadeing ontologies. "+e);
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		logger.debug("Ontologies found: "+ontologies.size());
		return ontologies;
	}


	/**
	 * read data from ResultSet into OlsOntology object
	 * @param rs
	 * @return
	 * @throws SQLException
	 */
	private OlsOntology getOntology(ResultSet rs) throws SQLException {
		OlsOntology ont = new OlsOntology();
		
	    ont.setDefinition(rs.getString("DEFINITION"));
	    ont.setFullName(rs.getString("FULLNAME"));
	    ont.setFullyLoaded(rs.getBoolean("FULLY_LOADED"));
	    ont.setLoadDate(rs.getDate("LOAD_DATE"));
	    ont.setShortName(rs.getString("SHORTNAME"));
	    ont.setVersion(rs.getString("VERSION"));
	    ont.setQueryURL(rs.getString("QUERY_URL"));
	    ont.setOntologyID(rs.getInt("ONTOLOGY_ID"));
	    
		return ont;
	}
	
	public List<OntologyRoot> getOntologyRoots() throws SQLException{
		
		Vector<OntologyRoot> roots = new Vector<OntologyRoot>();
		
		try {
			List<OlsOntology> ontologies =  getOntologies();
			
			for (OlsOntology ontology : ontologies) {
				List<OlsTerm> rootTerms = getRootTerms(ontology.getOntologyID());
				
				for (OlsTerm rootTerm : rootTerms) 
					roots.add(new OntologyRoot(ontology,rootTerm));
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw(e);
		}
		
		
		return roots;
	}


	/*
	 * 
	 */
	public List<OlsTerm> getRootTerms(Integer ontologyID) throws SQLException {
		List<OlsTerm> term = new Vector<OlsTerm>();
		
		ResultSet rs = null;
		PreparedStatement stmt = getStmtRootTermByOnt();
		try {
			stmt.setInt(1, ontologyID);
			rs = stmt.executeQuery();
			
			while(rs.next())
				term.add(getTerm(rs));
				
		} catch (SQLException e) {
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		return term;
	}


	protected final PreparedStatement getStmtOntologies() {
		return stmtOntologies;
	}


	protected PreparedStatement getStmtRootTermByOnt() {
		return stmtRootTermByOnt;
	}


	public OlsOntology getOntologyByID(Integer ontologyID) throws SQLException {
		PreparedStatement stmt = getStmtOntologyByID();
		
		
		ResultSet rs = null;
		try {
			stmt.setInt(1, ontologyID);
			rs = stmt.executeQuery();
			
			if (rs.next())
				return getOntology(rs);
			
			return null;
		} catch (SQLException e) {
			logger.error("Error while loading ontology by id.");
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
	}


	protected final PreparedStatement getStmtOntologyByID() {
		return stmtOntologyByID;
	}


	
	
	

}
